############################ PSQF 6272 Example 6b using R #####################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("Hmisc")) install.packages("Hmisc")
library(Hmisc) # To get bivariate correlations with p-values
# Note: has routine "describe" that competes with psych package

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lme4")) install.packages("lme4")
library(lme4) # To estimate MLMs using lmer

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC using lmer

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("emmeans")) install.packages("emmeans")
library(emmeans) # To get predicted means for factor variables

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

# Clear workspace (re-run as needed for troubleshooting purposes)
#rm(list = ls())

################################################################################
#####          BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 6a            #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/23_PSQF6272/PSQF6272_Example6b/"
filename = "Example6b_Data.xlsx"
setwd(dir=filesave)

# Load Jonathan's custom R functions from folder within working directory
functions = paste0("R functions/",dir("R functions/"))
temp = lapply(X=functions, FUN=source)

#  Import trimmed example excel data file from sheet "skin cancer"
Example6b = read_excel(paste0(filesave,filename), sheet="skin cancer") 
# Convert to data frame to use in analysis
Example6b = as.data.frame(Example6b)

# Open external file to save results to (turn off to use console instead)
txtStart(file=paste0(filesave,"Example6b_R_Output.txt"))

# Labels for original variables in comments only
#nation=    "nation: String-Format Nation" 
#regionID=  "regionID: L2 Region ID Variable"
#countyID=  "countyID: L1 County ID Variable"
#deaths=    "deaths: Count Outcome of Actual Deaths"
#expected=  "expected: Offset for Expected Deaths"
#uv=        "uv: County Amount of UV Exposure"

# Filter to only cases complete on all variables to be used below (before cluster means)
Example6b = Example6b[complete.cases(Example6b[ , c("regionID","deaths","uv")]),]

# Create region=cluster means using Jonathan's function
Example6b = addUnitMeans(data=Example6b, unitVariable="regionID",
                         meanVariables=c("deaths","uv"), newNames=c("RMdeaths","RMuv"))

print("R Descriptive Statistics within Region-Level-2 Data")
regionMeans = unique(Example6b[,c("regionID","NperregionID","RMdeaths","RMuv")])
describe(x=regionMeans[ , c("NperregionID","RMdeaths","RMuv")])
rcorr(x=as.matrix(regionMeans[ , c("NperregionID","RMdeaths","RMuv")]), type="pearson")

# Cluster-MC to get within-region deaths and UV
Example6b$WRdeaths = Example6b$deaths - Example6b$RMdeaths
Example6b$WRuv = Example6b$uv - Example6b$RMuv

print("R Descriptive Statistics within County-Level-1 Data")
describe(x=Example6b[ , c("deaths","WRdeaths","uv","WRuv")])
rcorr(x=as.matrix(Example6b[ , c("deaths","WRdeaths","uv","WRuv")]), type="pearson")
table(x=Example6b$nation, useNA="ifany")
# to save a plot: open a file, create the plot, then close the file
png(file = "R Deaths Histogram.png")  # open file
hist(x=Example6b$deaths, freq=FALSE,
     ylab="Density",xlab="deaths: Count Outcome of Actual Deaths") # axis labels
dev.off()  # close file


################################################################################
#####                 BEGIN EMPTY MODELS FOR EXAMPLE 6b                    #####
################################################################################

# To add an offset, add an offset= option instead glm or glmer
# glmer and glmer.nb are using laplace estimation below (quadrature broke it) 

print("R Empty Means, Random Intercept Model for UV Exposure (predictor)")
EmptyUV = lmer(data=Example6b, REML=TRUE, formula=uv~1+(1|regionID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(EmptyUV, chkREML=FALSE); summary(EmptyUV, ddf="Satterthwaite")
print("Show intraclass correlation and its LRT")
icc(EmptyUV); ranova(EmptyUV)

txtComment(" ") # insert blank space in output


print("R Model 1a: Empty Means, Single-Level for Deaths")
print("Log Link, Poisson Conditional Distribution")
Model1a = glm(data=Example6b, family=poisson(link="log"), formula=deaths~1)
-2*logLik(Model1a); summary(Model1a)
print("Convert log count to expected count via inverse link")
Model1aCount=exp(coefficients(Model1a)); Model1aCount 

txtComment(" ") # insert blank space in output


print("R Model 1b: Empty Means, Random Intercept for Deaths")
print("Log Link, Poisson Conditional Distribution, Laplace Estimation")
print("-2LL is not at all comparable, but results match SAS and STATA")
Model1b = glmer(data=Example6b, family=poisson(link="log"), 
                deaths~1+(1|regionID))
print("Show -2LL with precision, results, and LRT for random intercept")
-2*logLik(Model1b); summary(Model1b); anova(Model1b,Model1a)
print("Convert log count to expected count via inverse link")
Model1bCount=exp(fixef(Model1b)); Model1bCount 

print("ICC using icc function in performance package"); icc(Model1b)
print("Compute ICC manually using residual variance as defined below")
intvar=(Model1b@theta)^2; resvar=log(1+(1/exp(fixef(Model1b))))
ICC=intvar/(intvar+resvar); ICC

txtComment(" ") # insert blank space in output


print("R Model 1c: Empty Means, Random Intercept for Deaths")
print("Log Link, Negative Binomial Conditional Distribution")
Model1c = glmer.nb(data=Example6b, deaths~1+(1|regionID))
print("Show -2LL with precision, results, and LRT for overdispersion")
-2*logLik(Model1c); summary(Model1c); anova(Model1c,Model1b)
print("Convert log count to expected count via inverse link")
Model1cCount=exp(fixef(Model1c)); Model1cCount

print("Overdispersion from R")
getME(Model1c, "glmer.nb.theta")
print("Overdispersion that nearly matches STATA")
log(1/getME(Model1c, "glmer.nb.theta"))
print("Overdispersion that nearly matches SAS GLIMMIX")
(1/(getME(Model1c, "glmer.nb.theta")))

txtComment(" ") # insert blank space in output


################################################################################
#####           BEGIN CONDITIONAL MODELS FOR EXAMPLE 6b                    #####
################################################################################

print("R Model 2a:  Add Fixed Slope of Region Mean UV")
Model2a = glmer.nb(data=Example6b, deaths~1+RMuv+(1|regionID))
print("Show -2LL with precision, results, and overdispersion")
-2*logLik(Model2a); summary(Model2a); getME(Model2a, "glmer.nb.theta")
print("Get risk ratio effect size"); exp(fixef(Model2a))

txtComment(" ") # insert blank space in output


print("R Model 2b:  Add Fixed Slope of Within-Region UV")
Model2b = glmer.nb(data=Example6b, deaths~1+RMuv+WRuv+(1|regionID))
print("Show -2LL with precision, results, and overdispersion")
-2*logLik(Model2b); summary(Model2b); getME(Model2b, "glmer.nb.theta")
print("Get risk ratio effect sizes"); exp(getME(Model2b, "fixef"))
Model2bglht = summary(glht(model=Model2b, linfct=rbind(
  "UV Contextual Slope and Risk Ratio"= c(0,1,-1))))
Model2bglht; exp(Model2bglht$test$coefficients)

txtComment(" ") # insert blank space in output


print("R Model 2c:  Add Random Slope of Within-Region UV")
Model2c = glmer.nb(data=Example6b, deaths~1+RMuv+WRuv+(1+WRuv|regionID))
print("Show -2LL with precision, results, and overdispersion")
-2*logLik(Model2c); summary(Model2c); getME(Model2c, "glmer.nb.theta")
print("LRT for random slope"); anova(Model2c, Model2b)

txtComment(" ") # insert blank space in output


print("R Model 3a: Control for Main Effect of Nation on Intercept")
Model3a = glmer.nb(data=Example6b, deaths~1+RMuv+WRuv
                   +as.factor(nation)+(1+WRuv|regionID))
print("Show -2LL with precision, results, and overdispersion")
-2*logLik(Model3a); summary(Model3a); getME(Model3a, "glmer.nb.theta")
print("Omnibus DF=7 test of nation on intercept"); anova(Model3a) 
print("Predicted unit-specific log count by nation") 
emmeans(Model3a, specs="nation")

txtComment(" ") # insert blank space in output


print("R Model 3b: Add Nation Interactions with UV Slopes")
Model3b = glmer.nb(data=Example6b, deaths~1+RMuv+WRuv++as.factor(nation)
                   +as.factor(nation):RMuv+as.factor(nation):WRuv+(1+WRuv|regionID))
print("Show -2LL with precision, results, and overdispersion")
-2*logLik(Model3b); summary(Model3b); getME(Model3b, "glmer.nb.theta")
print("Omnibus DF=7 test of nation on intercept, each slope"); anova(Model3b) 
print("Simple slopes per country for between and within UV effects")
Model3bglht = glht(model=Model3b, linfct=rbind(
  "Between-Region UV: Belgium"    = c(0,1,0, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0),
  "Between-Region UV: Denmark"    = c(0,1,0, 0,0,0,0,0,0,0, 1,0,0,0,0,0,0, 0,0,0,0,0,0,0),
  "Between-Region UV: France"     = c(0,1,0, 0,0,0,0,0,0,0, 0,1,0,0,0,0,0, 0,0,0,0,0,0,0),
  "Between-Region UV: Ireland"    = c(0,1,0, 0,0,0,0,0,0,0, 0,0,1,0,0,0,0, 0,0,0,0,0,0,0),
  "Between-Region UV: Italy  "    = c(0,1,0, 0,0,0,0,0,0,0, 0,0,0,1,0,0,0, 0,0,0,0,0,0,0),
  "Between-Region UV: Netherlands"= c(0,1,0, 0,0,0,0,0,0,0, 0,0,0,0,1,0,0, 0,0,0,0,0,0,0),
  "Between-Region UV: UK"         = c(0,1,0, 0,0,0,0,0,0,0, 0,0,0,0,0,1,0, 0,0,0,0,0,0,0),
  "Between-Region UV: W Germany " = c(0,1,0, 0,0,0,0,0,0,0, 0,0,0,0,0,0,1, 0,0,0,0,0,0,0),
  "Within-Region UV: Belgium"    = c(0,0,1, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0),
  "Within-Region UV: Denmark"    = c(0,0,1, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 1,0,0,0,0,0,0),
  "Within-Region UV: France"     = c(0,0,1, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 0,1,0,0,0,0,0),
  "Within-Region UV: Ireland"    = c(0,0,1, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 0,0,1,0,0,0,0),
  "Within-Region UV: Italy  "    = c(0,0,1, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 0,0,0,1,0,0,0),
  "Within-Region UV: Netherlands"= c(0,0,1, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 0,0,0,0,1,0,0),
  "Within-Region UV: UK"         = c(0,0,1, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 0,0,0,0,0,1,0),
  "Within-Region UV: W Germany " = c(0,0,1, 0,0,0,0,0,0,0, 0,0,0,0,0,0,0, 0,0,0,0,0,0,1)))
summary(Model3bglht, test=adjusted("none"))


#######################################################################################
# Close external file of saved results
txtStop()


