// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

///////////////////////////////////////////////////////////////////////////////////////
////////               BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 6b         ///////
////////                  CHANGE "filesave" TO YOUR FOLDER PATH                 ///////
///////////////////////////////////////////////////////////////////////////////////////

// Define global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
   global filesave "C:\Dropbox\23_PSQF6272\PSQF6272_Example6b"

// Open trimmed example excel data file from sheet "skin cancer" and clear away existing data
   clear // clear memory in case of open data
   import excel "$filesave\Example6b_Data.xlsx", firstrow case(preserve) sheet("skin cancer") clear 
   
// Open external file to save results to
   log using $filesave\Example6b_STATA_Output.log, replace   

// Add labels to original variables
   label variable nation    "nation: String-Format Nation" 
   label variable regionID  "regionID: L2 Region ID Variable"
   label variable countyID  "countyID: L1 County ID Variable"
   label variable deaths    "deaths: Count Outcome of Actual Deaths"
   label variable expected  "expected: Offset for Expected Deaths"
   label variable uv        "uv: County Amount of UV Exposure"
	
// Select cases complete for analysis variables
   egen nmiss=rowmiss(region deaths uv nation)
   drop if nmiss>0

// Create cluster=regionID means
   egen regionN  = count(deaths), by(regionID)
   egen RMdeaths = mean(deaths),  by(regionID)
   egen RMuv     = mean(uv),      by(regionID)
   label variable regionN   "regionN: # Counties Contributing Data"
   label variable RMdeaths  "RMdeaths: Region Mean of Death Count"
   label variable RMuv      "RMuv: Region Mean of UV Exposure"

display "STATA Descriptive Statistics within Region-Level-2 Data"
preserve  // Save for later use, then compute region-level dataset
collapse  regionN RMdeaths RMuv, by(regionID)
summarize regionN RMdeaths RMuv
pwcorr    regionN RMdeaths RMuv, sig
restore   // Go back to county-level dataset

// Cluster-MC to get within-region deaths and UV
   gen WRdeaths = deaths - RMdeaths
   gen WRuv = uv - RMuv
   label variable WRdeaths  "WRdeaths: Within-regionID Deaths (0=RM)"
   label variable WRuv      "WRuv: Within-regionID UV Exposure (0=RM)"
	
// Convert nation from string to numeric
   encode nation, gen(catnation)

display "STATA Descriptive Statistics within County-Level-1 Data"
summarize deaths WRdeaths uv WRuv, detail
pwcorr    deaths WRdeaths uv WRuv, sig
tabulate  catnation
histogram deaths, percent discrete width(20) start(0)
graph export "$filesave\STATA Deaths Histogram.png", replace


////////////////////////////////////////////////////////////////////////////////
////////                BEGIN EMPTY MODELS FOR EXAMPLE 6b                ///////
////////////////////////////////////////////////////////////////////////////////

// To add an offset, use the exposure() option (which log-transforms it for you)
// I am using laplace estimation to try to match R results from glmer.nb
// To use quadrature instead, change option to intpoints(7) (7=STATA default)

display "STATA Empty Means, Random Intercept Model for UV Exposure (predictor)"
mixed uv , || regionID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
estat icc   // Intraclass Correlation

display "STATA Model 1a: Empty Means, Single-Level for Deaths"
display "Log Link, Poisson Conditional Distribution"
mepoisson deaths , nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model 
nlcom exp(_b[_cons])            // Fixed intercept in expected counts
             
display "STATA Model 1b: Empty Means, Random Intercept for Deaths"
display "Log Link, Poisson Conditional Distribution"
mepoisson deaths , || regionID: , intmethod(laplace) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model 
nlcom exp(_b[_cons])            // Fixed intercept in expected counts
estimates store Model1b         // Save for LRT
				 
display "STATA Model 1c: Empty Means, Random Intercept for Deaths"
display "Log Link, Negative Binomial Conditional Distribution"
menbreg deaths , || regionID: , intmethod(laplace) nolog coeflegend
display "-2LL = " e(ll)*-2      // Print -2LL for model 
nlcom exp(_b[_cons])            // Fixed intercept in expected counts
estimates store Model1c         // Save for LRT
lrtest Model1c Model1b          // LRT for overdispersion


////////////////////////////////////////////////////////////////////////////////
////////              BEGIN CONDITIONAL MODELS FOR EXAMPLE 6b            ///////
////////////////////////////////////////////////////////////////////////////////

display "STATA Model 2a: Add Fixed Slope of Region Mean UV"
menbreg deaths c.RMuv, || regionID: ,  intmethod(laplace) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model 
display "STATA Model 2a: Risk Ratio Effect Sizes Instead"
menbreg deaths c.RMuv, || regionID: , intmethod(laplace) nolog eform

display "STATA Model 2b: Add Fixed Slope of Within-Region UV"
menbreg deaths c.RMuv c.WRuv, || regionID: , intmethod(laplace) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model 
lincom c.RMuv*1 +c.WRuv*-1      // UV Contextual slope
estimates store Model2b         // Save for LRT     
display "STATA Model 2b: Risk Ratio Effect Sizes Instead"
menbreg deaths c.RMuv c.WRuv, || regionID: , intmethod(laplace) nolog eform
lincom c.RMuv*1 +c.WRuv*-1, irr   // UV Contextual slope
          
display "STATA Model 2c: Add Random Slope of Within-Region UV"
menbreg deaths c.RMuv c.WRuv, || regionID: c.WRuv, cov(un) intmethod(laplace) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model 
estimates store Model2c         // Save for LRT
lrtest Model2c Model2b          // LRT for random slope        
          
display "STATA Model 3a: Control for Main Effect of Nation on Intercept"
menbreg deaths c.RMuv c.WRuv i.catnation, ///
        || regionID: c.WRuv, cov(un) intmethod(laplace) nolog baselevels difficult
display "-2LL = " e(ll)*-2         // Print -2LL for model 
contrast i.catnation               // Omnibus DF=7 test of nation on intercept 
margins  i.catnation, predict(xb)  // Predicted unit-specific log count by nation
margins  i.catnation, predict(mu)  // Marginal expected count by nation
                    
display "STATA Model 3b: Add Nation Interactions with UV Slopes"
menbreg deaths c.RMuv c.WRuv i.catnation i.catnation#c.RMuv i.catnation#c.WRuv, ///
        || regionID: c.WRuv, cov(un) intmethod(laplace) nolog baselevels difficult
display "-2LL = " e(ll)*-2        // Print -2LL for model 
contrast i.catnation              // Omnibus DF=7 test catnation on intercept 
contrast i.catnation#c.RMuv       // Omnibus DF=7 test catnation on RMuv slope
contrast i.catnation#c.WRuv       // Omnibus DF=7 test catnation on WRuv slope
// UV Between fixed slope per catnation
margins, over(catnation) dydx(c.RMuv) predict(xb)
// UV Within fixed slope per catnation
margins, over(catnation) dydx(c.WRuv) predict(xb)

        
// Close external file of saved results
log close
